-- StubblePlough
-- Spec for StubblePlough to be used with chopped straw
-- When activated, the plough will work the soil but let the choppedStraw layer to have distinct visible behavior between Stubble Plough and Standard Plough
-- by yumi

source(Utils.getFilename("StubblePloughEvent.lua", g_currentModDirectory))

StubblePlough = {};

function StubblePlough.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(StubblePlough, specializations);
end;

function StubblePlough:load(xmlFile)
  self.setStubblePloughActive = SpecializationUtil.callSpecializationsFunction("setStubblePloughActive");
  self.isStubblePloughActive = false;
end;

function StubblePlough:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
  if not resetVehicles then
    local isStubblePloughActive= Utils.getNoNil(getXMLBool(xmlFile, key.."#StubblePlough"),false);

    if isStubblePloughActive == nil then
      self:setStubblePloughActive(false);
      --self.isStubblePloughActive = false;
    else
      self:setStubblePloughActive(isStubblePloughActive);
      --self.isStubblePloughActive = isStubblePloughActive;
    end

  end;
  return BaseMission.VEHICLE_LOAD_OK;
end;
 
function StubblePlough:getSaveAttributesAndNodes(nodeIdent)
  local attributes = 'StubblePlough="'.. tostring(self.isStubblePloughActive) .. '"';
  return attributes, nil;
end;

function StubblePlough:delete()
end;

function StubblePlough:readStream(streamId, connection)
  self:setStubblePloughActive(streamReadBool(streamId), true);
end;

function StubblePlough:writeStream(streamId, connection)
  streamWriteBool(streamId, self.isStubblePloughActive);
end;

function StubblePlough:mouseEvent(posX, posY, isDown, isUp, button)

end;

function StubblePlough:keyEvent(unicode, sym, modifier, isDown)
end;

function StubblePlough:onLeave()

end

function StubblePlough:update(dt)
  if self:getIsActiveForInput() and self.isClient then
    if InputBinding.hasEvent(InputBinding.OnOffStubblePlough) then
      self:setStubblePloughActive(not self.isStubblePloughActive);
    end
  end
end;

function StubblePlough:updateTick(dt)

  local preparingOutputId = nil;
  
  if self.ploughHasGroundContact and self:isLowered() then
    if g_currentMission.cs_strawBindings ~= nil and self.isStubblePloughActive then
      local fruitIdx = nil;
      
      local cx,cz;
      cx,_,cz = getWorldTranslation(self.rootNode);

      if cx ~= nil and cx==cx and cz==cz then -- Make extra sure that there actually is an x,z coordinate to use.
        local squareSize = 10
        local widthX,widthZ, heightX,heightZ = squareSize-0.5,0, 0,squareSize-0.5
        local x,z = cx - (squareSize/2), cz - (squareSize/2)
          
        -- Get current fruit on the groud
        local foundFruits = nil;
        for fruitIndex,fruit in pairs(g_currentMission.fruits) do
          if fruit.id ~= nil and fruit.id ~= 0 then

            setDensityCompareParams(fruit.id, "between", 2, 10)  -- #fruit from 1st growing state to withered state
            local _,numPixels,_ = getDensityParallelogram(fruit.id, x,z, widthX,widthZ, heightX,heightZ, 0,g_currentMission.numFruitStateChannels)
            setDensityCompareParams(fruit.id, "greater", 0)

            if numPixels > 0 then
              local fillTypeIndex = FruitUtil.fruitTypeToFillType[fruitIndex]
              local fillTypeName = FillUtil.fillTypeIndexToDesc[fillTypeIndex].nameI18N
              if fillTypeName == nil then
                fillTypeName = FillUtil.fillTypeIndexToDesc[fillTypeIndex].name
                if g_i18n:hasText(fillTypeName) then
                  fillTypeName = g_i18n:getText(fillTypeName)
                end
              end
              foundFruits = ((foundFruits == nil) and "" or foundFruits..", ") .. fillTypeName
              --print("We have found some " .. fillTypeName);
              --print("numPixels: " .. tostring(numPixels > 0))
              -- keep 1st one for now
              if fruitIdx == nil then
                fruitIdx = fruitIndex;
                break;
              end
            end
          end
        end
      end            
      
      if fruitIdx ~= nil then
        local fruitDesc = FruitUtil.fruitIndexToDesc[fruitIdx];
        local choppedStrawBinding = g_currentMission.cs_strawBindings[fruitDesc.name];
        if choppedStrawBinding ~= nil then
          local strawTypeFoliageId = g_currentMission.cs_strawTypes[choppedStrawBinding.strawTypeId].foliageId
          if strawTypeFoliageId ~= nil then
            for _,workArea in pairs(self.workAreas) do
              local x, _, z = getWorldTranslation(workArea.start)
              local x1, _, z1 = getWorldTranslation(workArea.width)
              local x2, _, z2 = getWorldTranslation(workArea.height)
              
              Utils.updateStrawArea(strawTypeFoliageId, x, z, x1, z1, x2, z2)
            end
          end
        end
      end;
    end
  end
end;

function StubblePlough:draw()
		--display a help message to say how to desactivate the lock
    if self.isStubblePloughActive then
      g_currentMission:addHelpButtonText(StubblePlough.l10nTexts["StubblePlough_Deactivate"], InputBinding.OnOffStubblePlough); --g_i18n:getText("StubblePlough_Deactivate"), InputBinding.OnOff);
    else
      g_currentMission:addHelpButtonText(StubblePlough.l10nTexts["StubblePlough_Activate"], InputBinding.OnOffStubblePlough); --g_i18n:getText("StubblePlough_Activate"), InputBinding.OnOff);
    end
end;

function StubblePlough:attachImplement(implement)
end;

function StubblePlough:setStubblePloughActive(isActive, noEventSend)
  self.isStubblePloughActive = isActive;
  if noEventSend ~= true then
    StubblePloughEvent.sendEvent(self, isActive);
  end;
end;
